/*
 *    Program type:  Interbase/Firebird/Isectd Interface
 *
 *    Description:
 *
 *    Executes unknown SQL statements
 *    returning the result to Isectd
 *    as a Python string capable of
 *    being evaluated (eval) by Python.
 *
 *
 * The contents of this file are subject to the Interbase Public
 * License Version 1.0 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy
 * of the License at http://www.Inprise.com/IPL.html
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code was created by Inprise Corporation
 * and its predecessors. Portions created by Inprise Corporation are
 * Copyright (C) Inprise Corporation.
 *
 * All Rights Reserved.
 * Contributor(s): Robert Kellock 24/07/2002
 */
/*
$Id: isectFirebird.c,v 1.2 2000/08/15 16:43:26 tmcsys Exp $
*/

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <time.h>
#include <ctype.h>
#include <ibase.h>
#include <isdio.h>
#include "align.h"
#include "example.h"

#define    MAXLEN               40000
#define    MAXMSG               1024

/* isectd declarations */
long bytes;
SOCKET sock;
isdHeader header;
char *msgBufferIn;
char msgBufferOut[MAXMSG];
int tran_active;


int process_statement (XSQLDA ISC_FAR * ISC_FAR * sqlda,
		       char ISC_FAR * msgBufferIn);
void print_column (XSQLVAR ISC_FAR * var);
long msg_input (int, long, char *);
void formatError(char *);

typedef struct vary
{
  short vary_length;
  char vary_string[1];
}
VARY;

isc_db_handle db = NULL;
isc_tr_handle trans = NULL;
isc_stmt_handle stmt = NULL;

/* Error message handling */
ISC_STATUS_ARRAY status;

#ifndef ISC_INT64_FORMAT

/* Define a format string for printf.  Printing of 64-bit integers
   is not standard between platforms */

#if (defined(_MSC_VER) && defined(WIN32)) || (defined(__BORLANDC__) && defined(__WIN32__))
#define	ISC_INT64_FORMAT	"I64"
#else
#define	ISC_INT64_FORMAT	"ll"
#endif
#endif


int
main (ARG (int, argc), ARG (char **, argv))
ARGLIST (int argc)
ARGLIST (char **argv)
{
  /* firebird declarations */
  XSQLDA ISC_FAR *sqlda;
  char *db_name;
  char *user_name;
  char *user_passwd;
  char *charset;
  char *role;
  char *dpb;
  char dpb_buffer[256];
  short dpb_length;

  if (argc < ISDARGMAX + 3)
    {
      printf ("You must have three arguments: database  username  password\n\
Two additional arguments that you can have are: charset  role\n");
      ERREXIT (status, 1);
    }

  db_name = argv[1];
  user_name = argv[2];
  user_passwd = argv[3];
  if (argc >= ISDARGMAX + 4)
    charset = argv[4];
  else
    charset = NULL;
  if (argc >= ISDARGMAX + 5)
    role = argv[5];
  else
    role = NULL;

  dpb = dpb_buffer;
  *dpb = isc_dpb_version1;
  dpb_length = 1;

  if (user_name)
    isc_expand_dpb (&dpb, (short *) &dpb_length, isc_dpb_user_name, user_name,
		    NULL);

  if (user_passwd)
    isc_expand_dpb (&dpb, (short *) &dpb_length, isc_dpb_password,
		    user_passwd, NULL);

  if (charset)
    isc_expand_dpb (&dpb, (short *) &dpb_length, isc_dpb_lc_ctype, charset,
		    NULL);

  if (role)
    isc_expand_dpb (&dpb, (short *) &dpb_length, isc_dpb_sql_role_name, role,
		    NULL);

  if (isc_attach_database
      (status, (short) strlen (db_name), db_name, &db, dpb_length, dpb))
    {
      printf ("Could not open database %s\n", db_name);
      ERREXIT (status, 1);
    }

  /*
     *    Allocate enough space for 20 fields.
     *    If more fields get selected, re-allocate SQLDA later.
   */
  sqlda = (XSQLDA ISC_FAR *) malloc (XSQLDA_LENGTH (20));
  sqlda->sqln = 20;
  sqlda->version = 1;

  /* Allocate a global statement */
  if (isc_dsql_allocate_statement (status, &db, &stmt))
    {
      free (sqlda);
    ERREXIT (status, 1)}

  /* isectd event loop begins */
  if (argc >= ISDARGMAX)
    {
        //if ((sock = isdConnect (argv)) != -1)
	if ((sock = isdAttach (argv[argc-3], argv[argc-2], argv[argc-1], 0)) != -1)
	{
	  for (;;)
	    {
	      if ((msgBufferIn = isdRecv (sock, &header)))
		{

		  /*
		   *    Process SQL statements.
		   */
		  isdLog ("msgBufferIn = %s, length = %d, more = %d\n",
			  msgBufferIn, header.len, header.more);
		  bytes = 0;

		  if (header.len)
		    {
		      /* save incoming header.more instruction for later */
		      tran_active = header.more;
		      msgBufferIn[header.len + 1] = '\0';
		      if (!strncmp (msgBufferIn, "commit", 6))
			{
			  isdLog ("Committing - OK\n");
			  if (trans)
			    if (isc_commit_transaction (status, &trans))
			      ERREXIT (status, 2);
			  /* echo commit to client, which is unlikely
			     to be listening for a reply anyway, to free worker */
			  header.more = 0;
			  bytes =
			    isdSend (sock, msgBufferIn, header.len, &header);
			}
		      else if (!strncmp (msgBufferIn, "rollback", 8))
			{
			  /* transaction aborted and rolled back because there has been a Zope error */
			  isdLog ("Rolling back - Zope error\n");
			  if (trans)
			    if (isc_rollback_transaction (status, &trans))
			      ERREXIT (status, 2);
			  /* echo rollback to client, which is unlikely
			     to be listening for a reply anyway, to free worker */
			  header.more = 0;
			  bytes =
			    isdSend (sock, msgBufferIn, header.len, &header);
			}
		      else
			{
			  if (!process_statement
			      ((XSQLDA ISC_FAR * ISC_FAR *) & sqlda,
			       (char ISC_FAR *) msgBufferIn))
			    {
			      if (tran_active)
				header.more = -1;
			      else
				header.more = 0;
			    }
			  else
			    {
			      isdLog ("Rolling back - SQL error\n");
			      if (trans)
				if (isc_rollback_transaction (status, &trans))
				  ERREXIT (status, 2);
			      header.more = 0;
			    }
			  /* Final chunk of query result returned to Isectd. */
			  header.len = bytes;
			  bytes =
			    isdSend (sock, msgBufferOut, header.len, &header);
              isdLog ("final msgBufferOut = %s, length = %d, more = %d\n",
			  msgBufferOut, header.len, header.more);  
			}
		    }
		  else
		    /* Echo back empty message */
		    {
		      header.more = 0;
		      bytes =
			isdSend (sock, msgBufferIn, header.len, &header);
		    }
		  if (bytes < 0)
		    isdError ("%s: isdSend() reply\n", argv[ISDARGWID],
			      strerror (errno));
		}
	      else
		break;
	    }
	  isdLogout (sock);
	}
      else
	isdError ("isdConnect(): %s\n", strerror (errno));
    }
  else
    fprintf (stderr, "Incorrect number of arguments\n");

  free (sqlda);
  if (trans)
    if (isc_commit_transaction (status, &trans))
      {
	ERREXIT (status, 1);
      }

  if (isc_detach_database (status, &db))
    {
      ERREXIT (status, 1);
    }

  return 0;
}

/*
**  Process submitted statement. On any fundamental error, return status 1,
**  which will do an isc_print_status and exit the program.
**  On user errors, found in parsing or executing go to formatError which rolls
**  back the current transaction and return the error message to Isectd.
**  Status 2 means nothing but we will use it to indicate that the error will be
**  printed to stderr. However, the Isectd loop will continue.
*/

int
process_statement (ARG (XSQLDA ISC_FAR * ISC_FAR *, sqldap),
		   ARG (char ISC_FAR *, query))
ARGLIST (XSQLDA ** sqldap)
ARGLIST (char *query)
{
  long buffer[MAXLEN];
  XSQLDA ISC_FAR *sqlda;
  XSQLVAR ISC_FAR *var;
  short num_cols, i;
  short type, null_ok;
  long length, offset, alignment;
  int fetch_stat; // W. Robert Kellock 2015-08-15 used to be long
  static char stmt_info[] = { isc_info_sql_stmt_type };
  char info_buffer[20];
  short l;
  long statement_type = 0;
  char info_holder[42], *info_type;
  long row_count = 0;

  sqlda = *sqldap;

  /* Start a transaction if we are not in one */
  if (!trans)
    if (isc_start_transaction (status, &trans, 1, &db, 0, NULL))
      {
      ERREXIT (status, 2)}

  if (isc_dsql_prepare
      (status, &trans, &stmt, 0, query, SQL_DIALECT_V6, sqlda))
    {
      formatError("isc_dsql_prepare");
      return -1;
    }

  /* What is the statement type of this statement?
     **
     ** stmt_info is a 1 byte info request.  info_buffer is a buffer
     ** large enough to hold the returned info packet
     ** The info_buffer returned contains a isc_info_sql_stmt_type in the first byte,
     ** two bytes of length, and a statement_type token.
   */
  if (!isc_dsql_sql_info (status, &stmt, sizeof (stmt_info), stmt_info,
			  sizeof (info_buffer), info_buffer))
    {
      l = (short) isc_vax_integer ((char ISC_FAR *) info_buffer + 1, 2);
      statement_type = isc_vax_integer ((char ISC_FAR *) info_buffer + 3, l);
    }


  /*
   *    Execute a non-select statement.
   */
  if (!sqlda->sqld){
      if (isc_dsql_execute (status, &trans, &stmt, SQL_DIALECT_V6, NULL)){
	     formatError("isc_dsql_execute 1");
	     return -1;
	     }
      else{
           /* Commit DDL statements if that is what sql_info says */
           if (trans && (statement_type == isc_info_sql_stmt_ddl)){
   	          printf ("\tCommitting...\n");
	          if (isc_commit_transaction (status, &trans)){
		     formatError("isc_commit_transaction");
	             return -1;
	             }
           }
	       bytes = msg_input (0, 0, "[[],[]]");
	       }
      return 0;
  }

  /*
   *    Process select statements.
   */

  num_cols = sqlda->sqld;

  /* Need more room. */
  if (sqlda->sqln < num_cols)
    {
      *sqldap = sqlda = (XSQLDA ISC_FAR *) realloc (sqlda,
						    XSQLDA_LENGTH (num_cols));
      sqlda->sqln = num_cols;
      sqlda->version = 1;

      if (isc_dsql_describe (status, &stmt, SQL_DIALECT_V6, sqlda))
	{
          formatError("isc_dsql_describe");
	  return -1;
	}

      num_cols = sqlda->sqld;
    }

    /* 2002/08/02 Robert Kellock moved error checking on execute, above SQLDA setup
       rather than below it, to catch multiple rows in singleton select type errors
    */

    if (isc_dsql_execute (status, &trans, &stmt, SQL_DIALECT_V6, NULL))
    {
      formatError("isc_dsql_execute 2");
      return -1;
    }

  /*
   *     Set up SQLDA.
   */
  bytes = msg_input (0, 0, "[[");
  for (var = sqlda->sqlvar, offset = 0, i = 0; i < num_cols; var++, i++)
    {
      if (i > 0)
	bytes = msg_input (0, 0, "\x2C");
      bytes = msg_input (0, 0, "{");
      length = alignment = var->sqllen;
      type = var->sqltype & ~1;
      null_ok = var->sqltype & 1;

      /* Set up Python data dictionary */
      sprintf (info_holder, "\'name\':\'%s\'\x2C", var->aliasname);
      bytes = msg_input (0, 0, info_holder);
      switch (type)
	{
	case SQL_TEXT:
	case SQL_VARYING:
	case SQL_BLOB:
	case SQL_ARRAY:
	  info_type = "s";
	  break;
	case SQL_TIMESTAMP:
	case SQL_TYPE_TIME:
	case SQL_TYPE_DATE:
	  info_type = "d";
	  break;
	case SQL_SHORT:
	case SQL_LONG:
	case SQL_INT64:
	  info_type = "i";
	  break;
	case SQL_FLOAT:
	case SQL_DOUBLE:
	  info_type = "n";
	  break;
	default:
	  info_type = "?";	/* this type should never exist */
	  break;
	}
      sprintf (info_holder, "\'type\':\'%s\'\x2C", info_type);
      bytes = msg_input (0, 0, info_holder);
      sprintf (info_holder, "\'null\':%d\x2C", null_ok);
      bytes = msg_input (0, 0, info_holder);
      sprintf (info_holder, "\'width\':%ld}", length);
      bytes = msg_input (0, 0, info_holder);

      if (type == SQL_TEXT)
	alignment = 1;
      else if (type == SQL_VARYING)
	{
	  length += sizeof (long) + 1;
	  alignment = sizeof (long);
	}
      /*  RISC machines are finicky about word alignment
         **  So the output buffer values must be placed on
         **  word boundaries where appropriate
       */
      offset = FB_ALIGN (offset, alignment);
      var->sqldata = (char ISC_FAR *) buffer + offset;
      offset += length;
      offset = FB_ALIGN (offset, sizeof (long));
      var->sqlind = (short *) ((char ISC_FAR *) buffer + offset);
      offset += sizeof (long);
    }
  bytes = msg_input (0, 0, "]\x2C\n[");


  /*
   *    Print rows.
   */
  while ((fetch_stat =
	  isc_dsql_fetch (status, &stmt, SQL_DIALECT_V6, sqlda)) == 0)
    {
      if (row_count > 0)
            bytes = msg_input (0, 0, "\x2C\n");
      row_count += 1;
      bytes = msg_input (0, 0, "[");
      for (i = 0; i < num_cols; i++)
	{
	  if (i > 0)
	    bytes = msg_input (0, 0, "\x2C");
	  print_column ((XSQLVAR ISC_FAR *) & sqlda->sqlvar[i]);
	}
      bytes = msg_input (0, 0, "]");
      /* bytes = msg_input (0, 0, "]\x2C\n"); */
    }
  bytes = msg_input (0, 0, "]]\n");

  /* W. Robert Kellock 2015-08-15 Moved fetch_stat above isc_dsql_free_statement rather
     than below it and removed the return call to catch exceptions generated by triggers 
     called by a select statement.  But since we have already sent the SQLDA to msg_input
     we are going to have to clear out what we've already sent and replace it with the
     error message. 
  */

  if (fetch_stat != 100L)
    {
      formatError("fetch_stat");
      // return -1;
    }

  /* Close cursor */
  if (isc_dsql_free_statement (status, &stmt, DSQL_close))
    {
      formatError("isc_dsql_free_statement");
      return -1;
    };

  return 0;
}

/*
 *    Print column's data.
 */
void
print_column (ARG (XSQLVAR ISC_FAR *, var))
ARGLIST (XSQLVAR * var)
{
  short dtype;
  char data[MAXLEN], *p;
  char blob_s[20], date_s[25];
  VARY *vary;
  struct tm times;
  ISC_QUAD bid;

  dtype = var->sqltype & ~1;
  p = data;

  /* Null handling.  If the column is nullable and null */
  if ((var->sqltype & 1) && (*var->sqlind < 0))
    {
      sprintf (p, "%s", "None");
      bytes = msg_input (1, 0, p);
    }
  else
    { /*
	2015-12-27	W. Robert Kellock	All of the sprintf functions have been altered to simplify the
						format specifiers to limit the possibility of trailing spaces.
      */
      switch (dtype)
	{
	case SQL_TEXT:
	  sprintf (p, "%.*s", var->sqllen, var->sqldata);
	  break;

	case SQL_VARYING:
	  vary = (VARY *) var->sqldata;
	  vary->vary_string[vary->vary_length] = '\0';
	  sprintf (p, "%-*s", vary->vary_length, vary->vary_string);
	  break;

	case SQL_SHORT:
	case SQL_LONG:
	case SQL_INT64:
	  {
	    ISC_INT64 value = 0;
	    short field_width = 0;
	    short dscale;
	    switch (dtype)
	      {
	      case SQL_SHORT:
		value = (ISC_INT64) * (short ISC_FAR *) var->sqldata;
		field_width = 6;
		break;
	      case SQL_LONG:
		value = (ISC_INT64) * (long ISC_FAR *) var->sqldata;
		field_width = 11;
		break;
	      case SQL_INT64:
		value = (ISC_INT64) * (ISC_INT64 ISC_FAR *) var->sqldata;
		field_width = 21;
		break;
	      }
	    dscale = var->sqlscale;
	    if (dscale < 0)
	      {
		ISC_INT64 tens;
		short i;

		tens = 1;
		for (i = 0; i > dscale; i--)
		  tens *= 10;

		if (value >= 0)
		  sprintf (p,
			   "%*" ISC_INT64_FORMAT "d.%0*"
			   ISC_INT64_FORMAT "d",
			   field_width - 1 + dscale,
			   (ISC_INT64) value / tens, -dscale,
			   (ISC_INT64) value % tens);
		else if ((value / tens) != 0)
		  sprintf (p,
			   "%*" ISC_INT64_FORMAT "d.%0*"
			   ISC_INT64_FORMAT "d",
			   field_width - 1 + dscale,
			   (ISC_INT64) (value / tens), -dscale,
			   (ISC_INT64) - (value % tens));
		else
		  sprintf (p, "%*s.%0*" ISC_INT64_FORMAT "d",
			   field_width - 1 + dscale,
			   "-0", -dscale, (ISC_INT64) - (value % tens));
	      }
	    else if (dscale)
	      sprintf (p, "%*" ISC_INT64_FORMAT "d%0*d",
		       field_width, (ISC_INT64) value, dscale, 0);
	    else
	      sprintf (p, "%*" ISC_INT64_FORMAT "d",
		       field_width, (ISC_INT64) value);
	  };
	  break;

	case SQL_FLOAT:
	  sprintf (p, "%15g", *(float ISC_FAR *) (var->sqldata));
	  break;

	case SQL_DOUBLE:
	  sprintf (p, "%24f", *(double ISC_FAR *) (var->sqldata));
	  break;

	case SQL_TIMESTAMP:
	  isc_decode_timestamp ((ISC_TIMESTAMP ISC_FAR *) var->sqldata,
				&times);
	  sprintf (date_s, "%04d-%02d-%02d %02d:%02d:%02d.%04ld",
		   times.tm_year + 1900, times.tm_mon + 1, times.tm_mday,
		   times.tm_hour, times.tm_min, times.tm_sec,
		   ((ISC_TIMESTAMP *) var->sqldata)->timestamp_time % 10000);
	  sprintf (p, "%*s", 24, date_s);
	  break;

	case SQL_TYPE_DATE:
	  isc_decode_sql_date ((ISC_DATE ISC_FAR *) var->sqldata, &times);
	  sprintf (date_s, "%04d-%02d-%02d", times.tm_year + 1900,
		   times.tm_mon + 1, times.tm_mday);
	  sprintf (p, "%*s", 10, date_s);
	  break;

	case SQL_TYPE_TIME:
	  isc_decode_sql_time ((ISC_TIME ISC_FAR *) var->sqldata, &times);
	  sprintf (date_s, "%02d:%02d:%02d.%04ld", times.tm_hour,
		   times.tm_min, times.tm_sec,
		   (*((ISC_TIME *) var->sqldata)) % 10000);
	  sprintf (p, "%*s", 13, date_s);
	  break;

	case SQL_BLOB:
	case SQL_ARRAY:
	  /* Print the blob id on blobs or arrays */
	  bid = *(ISC_QUAD ISC_FAR *) var->sqldata;
	  sprintf (blob_s, "%08ld:%08ld", bid.isc_quad_high,
		   bid.isc_quad_low);
	  sprintf (p, "%17s", blob_s);
	  break;

	default:
	  break;
	}
      if ((dtype == SQL_TEXT) || (dtype == SQL_VARYING) || (dtype == SQL_TIMESTAMP) || (dtype == SQL_TYPE_DATE) || (dtype == SQL_TYPE_TIME))
	{
	  bytes = msg_input (0, 0, "\'");
	  if ((dtype == SQL_TEXT) || ((dtype == SQL_VARYING) && (vary->vary_length > 0)) || (dtype == SQL_TIMESTAMP) || (dtype == SQL_TYPE_DATE) || (dtype == SQL_TYPE_TIME))
	  	bytes = msg_input (1, 0, p);
	  bytes = msg_input (0, 0, "\'");
	}
      else
 	  bytes = msg_input (1, 0, p);
    }
}

/*
        msg_input concatenates a variable length string onto a fixed length string.
        If the variable length string is longer than the fixed length string or
        concatenating the variable length string to the fixed length string would
        exceed it, the contents of the fixed length string is sent to Isectd and the
        remainder of the variable length string is concatenated recursively onto
        the beginning of the fixed length string.
*/
long
msg_input (int escape, long pos, char *input)
{
  /* pos used to keep track of position in char *input */
  long k = strlen (input);
  long j = bytes;
  if (pos >= k) return j; //Prevent an octal conversion at the end of a field causing a buffer over-run.
  /*
	2015-12-27	W. Robert Kellock	Because we have removed any possibility of trailing spaces,
						all the code related to removing trailing spaces has been 
						removed.
  */
  for (j; j < MAXMSG - 1; j++)
    {
      /* isdLog("input[%ld] = %x\n",j,input[pos]); */
      if (escape)
	{
	  /* Convert to Python octal escape characters */
	  switch (input[pos])
	    {
	      /* Null bytes (\\000) indicate a buffer over-run! */
	    case '\x00':
	      bytes = j;
	      bytes = msg_input (0, 0, "\\000");
	      return msg_input (1, pos + 1, input);
	      break;
	    case '\x0a':
	      bytes = j;
	      bytes = msg_input (0, 0, "\\012");
	      return msg_input (1, pos + 1, input);
	      break;
	    case '\x0d':
	      bytes = j;
	      bytes = msg_input (0, 0, "\\015");
	      return msg_input (1, pos + 1, input);
	      break;
	    case '\x22':
	      bytes = j;
	      bytes = msg_input (0, 0, "\\042");
	      return msg_input (1, pos + 1, input);
	      break;
	    case '\x27':
	      bytes = j;
	      bytes = msg_input (0, 0, "\\047");
	      return msg_input (1, pos + 1, input);
	      break;
	    case '\x2C':
	      bytes = j;
	      bytes = msg_input (0, 0, "\\054");
	      return msg_input (1, pos + 1, input);
	      break;
	    case '\x28':
	      bytes = j;
	      bytes = msg_input (0, 0, "\\050");
	      return msg_input (1, pos + 1, input);
	      break;
	    case '\x29':
	      bytes = j;
	      bytes = msg_input (0, 0, "\\051");
	      return msg_input (1, pos + 1, input);
	      break;
	    case '\x5B':
	      bytes = j;
	      bytes = msg_input (0, 0, "\\133");
	      return msg_input (1, pos + 1, input);
	      break;
	    case '\x5D':
	      bytes = j;
	      bytes = msg_input (0, 0, "\\135");
	      return msg_input (1, pos + 1, input);
	      break;
	    case '\x7B':
	      bytes = j;
	      bytes = msg_input (0, 0, "\\173");
	      return msg_input (1, pos + 1, input);
	      break;
	    case '\x7D':
	      bytes = j;
	      bytes = msg_input (0, 0, "\\175");
	      return msg_input (1, pos + 1, input);
	      break;
	    }
	}
      msgBufferOut[j] = input[pos];
      pos++;
      if (pos >= k)
	break;
    }

  if (pos < k)
    /* if we are here then we have exceeded the capacity of msgBufferOut
       and must start a new one recursively */
    {

      msgBufferOut[MAXMSG - 1] = '\0';
      /* isdLog("msgBuffer sent = %s\n",msgBufferOut); */

      header.len = MAXMSG - 1;
      header.more = 1;
      bytes = isdSend (sock, msgBufferOut, header.len, &header);
      bytes = 0;
      return msg_input (escape, pos, input);
    }
  else
    /* global int bytes used to keep track of msgBufferOut contents.
       If we are here then there is still free space available */
    {
      /* msgBufferOut[j+1] = '\0'; */
      /* isdLog("msgBuffer concat = %s\n",msgBufferOut); */
      return (j + 1);
    }

}

void formatError(char origin[48]){
/* 
	2004-12-20  W. Robert Kellock
	All Firebird errors now processed using isc_interprete which
	understands user defined SQL exceptions as well as system errors

	2010-01-19  W. Robert Kellock
	Replaced deprecated isc_interprete with fb_interpret

	2015-08-15  W. Robert Kellock
	fb_interpret not being called correctly

*/
	ISC_SCHAR error_msg[MAXMSG];
	ISC_STATUS* pVector = (ISC_STATUS*)status;
	bytes = 0;// If there is an error we need to start again and overwrite any recordset that has already been written 
	while (fb_interpret(error_msg, sizeof(error_msg), (const ISC_STATUS**)&pVector)){
		bytes = msg_input (0, 0, error_msg);
		bytes = msg_input (0, 0, "\n");
	}
	isdLog("formatError origin = %s\n",origin);
}
