/*
  scriptworker.c - proxy executable caller
  W. Robert Kellock 2003/04/14 
  
  lets Isectd call a scripting language worker where the scripting
  language environment requires its own command line arguments 
  which will not be passed into the script itself eg. Progress 
  
  creates a new command line from argv[5] onwards
  substituting argv[1], argv[2] and argv[3]
  
  eg.  C:\scriptworker.exe localhost 1134 1 -sprogress C:\Progress\Prowin32.exe -p test.p -param "argv[1] argv[2] argv[3] mydatabase mypassword"
       becomes
       C:\Progress\Prowin32.exe -p test.p -param "localhost 1134 1 mydatabase mypassword"

  Notice that the service name (-sprogress) is not needed for workers. Instead the port number (1134) is substituted for the
  service name.
*/

//#ifdef _Windows
       #include <windows.h>
       #include <tchar.h>
//#endif

#include <string.h>
#include <stdio.h>

#define BUFSIZE 1024

int windows_system(const char *cmd)
{
  PROCESS_INFORMATION p_info;
  STARTUPINFO s_info;
  LPSTR cmdline;

  memset(&s_info, 0, sizeof(s_info));
  memset(&p_info, 0, sizeof(p_info));
  s_info.cb = sizeof(s_info);

  cmdline = _tcsdup(TEXT(cmd));

//  if (CreateProcess(NULL, cmdline, NULL, NULL, 0, CREATE_NO_WINDOW, NULL, NULL, &s_info, &p_info))
  if (CreateProcess(NULL, cmdline, NULL, NULL, 0, 0, NULL, NULL, &s_info, &p_info))
  {
    WaitForSingleObject(p_info.hProcess, INFINITE);
    CloseHandle(p_info.hProcess);
    CloseHandle(p_info.hThread);
    return 1;
  }
  return -1;
}

int
main (int argc, char **argv)
{
  char buf[BUFSIZE], vsbuf[BUFSIZE] = {};
  int i,j,m,n,q,r;

  if (argc<5){
     printf("scriptworker expects at least 5 command line arguments.\n");
     return 0;
     }

  printf("\nStarting scriptworker proxy executable...\n\n");
  printf("command line hidden = %s %s %s %s %s\n", argv[0], argv[1], argv[2], argv[3], argv[4]);
  q=5;
  for (i=q; i<argc; i++){
      printf("argv[%d] = %s\n",i,argv[i]);
      n=r=0;
      memset(buf,0, BUFSIZE);
      for (j=0; j<strlen(argv[i]); j++){
          if (n){
             /* Null, Tab or Space */
             while (argv[i][j]==0||argv[i][j]==0x0009||argv[i][j]==0x0020){
                   r=1;
                   j++;
                   }
             n=0;
      	     memset(buf,0, BUFSIZE);
             }
          strncat(buf,&argv[i][j],1);
          if (!strcmp(buf,"argv[1]")){
             n=1;
             if (r)
                strcat(vsbuf," ");
/*
	     else
      		strcat(vsbuf,"\"");
*/
             strcat(vsbuf,argv[1]); 
             }
          else if (!strcmp(buf,"argv[2]")){
             n=1;
             if (r){
                strcat(vsbuf," ");
                strcat(vsbuf,argv[2]); 
                }
               }
          else if (!strcmp(buf,"argv[3]")){
             n=1;
             if (r){
                strcat(vsbuf," ");
                strcat(vsbuf,argv[3]); 
                }
	}
      }
	if(!n){
         if (r)
            strcat(vsbuf," ");
	 strcat(vsbuf,buf);
         if (!r)
            strcat(vsbuf," ");	
      }
}
      //strcat(vsbuf,"\"");
      printf("\ncommand line rearranged = %s\n\n", vsbuf);

//#ifdef _Windows
     if(windows_system(vsbuf)<0)
     	printf("scriptworker failed to start third party process.\n");
     else
     	printf("scriptworker completed.\n");
//#else
//     execvp(argv[0], vsbuf);
//#endif

  return 0;
}
