/*************************************************************************
* CommandLineToArgvWR
*
* We must interpret the quotes in the command line to rebuild the argv
* array correctly:
* - arguments are separated by spaces or tabs
* - quotes serve as optional argument delimiters
*   '"a b"'   -> 'a b'
* - escaped quotes must be converted back to '"'
*   '\"'      -> '"'
* - an odd number of '\'s followed by '"' correspond to half that number
*   of '\' followed by a '"' (extension of the above)
*   '\\\"'    -> '\"'
*   '\\\\\"'  -> '\\"'
* - an even number of '\'s followed by a '"' correspond to half that number
*   of '\', plus a regular quote serving as an argument delimiter (which
*   means it does not appear in the result)
*   'a\\"b c"'   -> 'a\b c'
*   'a\\\\"b c"' -> 'a\\b c'
* - '\' that are not followed by a '"' are copied literally
*   'a\b'     -> 'a\b'
*   'a\\b'    -> 'a\\b'
*
* Note:
* '\t' == 0x0009
* ' '  == 0x0020
* '"'  == 0x0022
* '\\' == 0x005c
*
*
* W Robert Kellock 2002/10/09  this routine has been
* copied from the Wine project www.winehq.com and modified
* to accept ASCII characters only. It will not work with
* UNICODE
*
*/
 
#include <windows.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
 
LPTSTR* WINAPI CommandLineToArgvWR(char* lpCmdline, int* numargs)
{
     DWORD argc,argc_holder;
     HGLOBAL hargv;
     LPTSTR  *argv;
     LPTSTR cs;
     LPTSTR arg,s,d;
     LPTSTR cmdline;
     int in_quotes,bcount;
     
     /* to get a writeable copy */
     argc_holder=1;
     bcount=0;
     in_quotes=0;
     cs=lpCmdline;
     while (1) {
         if (*cs==0 || ((*cs==0x0009 || *cs==0x0020) && !in_quotes)) {
             /* end of command line */
             if (*cs==0)
                break;
             /* space */
             argc_holder++;
             /* skip the remaining spaces */
             while (*cs==0x0009 || *cs==0x0020)
                 cs++;
             /* end of command line with spaces beforehand */
             if (*cs==0){
                argc_holder--;
                break;
                }
             else
                cs--;
             bcount=0;
         } else if (*cs==0x005c) {
             /* '\', count them */
             bcount++;
         } else if ((*cs==0x0022) && ((bcount & 1)==0)) {
             /* unescaped '"' */
             in_quotes=!in_quotes;
             bcount=0;
         } else {
             /* a regular character */
             bcount=0;
         }
         cs++;
     }
     /* Allocate in a single lump, the string array, and the strings that go with it.
      * This way the caller can make a single GlobalFree call to free both, as per MSDN.
      */
     hargv=GlobalAlloc(0, argc_holder*sizeof(LPTSTR)+(strlen(lpCmdline)+1)*sizeof(CHAR));
     argv=GlobalLock(hargv);
     if (!argv)
         return NULL;
     cmdline=(LPTSTR)(argv+argc_holder);
     strcpy(cmdline, lpCmdline);
 
     argc=0;
     bcount=0;
     in_quotes=0;
     arg=d=s=cmdline;
     while (*s) {
         if ((*s==0x0009 || *s==0x0020) && !in_quotes) {
             /* Close the argument and copy it */
             *d=0;
             argv[argc++]=arg;
 
             /* skip the remaining spaces */
             do {
                 s++;
             } while (*s==0x0009 || *s==0x0020);
 
             /* Start with a new argument */
             arg=d=s;
             bcount=0;
         } else if (*s==0x005c) {
             /* '\\' */
             *d++=*s++;
             bcount++;
         } else if (*s==0x0022) {
             /* '"' */
             if ((bcount & 1)==0) {
                 /* Preceeded by an even number of '\', this is half that
                  * number of '\', plus a quote which we erase.
                  */
                 d-=bcount/2;
                 in_quotes=!in_quotes;
                 s++;
             } else {
                 /* Preceeded by an odd number of '\', this is half that
                  * number of '\' followed by a '"'
                  */
                 d=d-bcount/2-1;
                 *d++='"';
                 s++;
             }
             bcount=0;
         } else {
             /* a regular character */
             *d++=*s++;
             bcount=0;
         }
     }
     if (*arg) {
         *d='\0';
         argv[argc]=arg;
     }
     if (numargs)
        *numargs=argc_holder;    
 
     return argv;
 }

