/*
2014-03-01  W. Robert Kellock - major clean up of isdClient begun.
*/ 
    
#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif
  
#include "../isd/isdio.h" /* W Robert Kellock 2002/08/09  hard path */
  
#include <malloc.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
  
#ifndef _Windows        /* W Robert Kellock 2002/08/09  added Windows switch */
	#include <sys/time.h>
	#include <sys/types.h>
#else	
	#include <windows.h>
	#include <conio.h>
	#include "guicon.h"
#endif
  
#ifdef HAVE_W3C_LIBWWW_WWWLIB_H
	#include "w3c-libwww/WWWLib.h"
	#ifdef LIBWWW_SHARED
		#include "HTextImp.h"
	#endif	
#endif
  
#define ISDCLIENT_BUFSIZE 4096
/* W Robert Kellock 2002/08/15 added directive test */ 
#ifndef max
	#define max(a, b) (a) > (b) ? (a):(b)
#endif
  
static struct
{
SOCKET sock;
int keep;
short priority;         /* W Robert Kellock 2003/11/27 Priority now a flag */
int unescapeURLs;
int debug;
// int sendOnEOF;  	... I don't think this is needed anymore as we are reading into a file buffer now!!!
int reply;
int runOneCommand;
FILE * inputFile;
char *command;
char *host;
char *service;
short guiX;		/* W Robert Kellock 2002/10/14 gui console switch which also specifies console width */
}
isdClientData;

char *parseCommandLine(char *str){
   /* 
	Convert \n's anywhere in command line arguments to newlines
   */
   int i, found, firstTime = 1;
   int strLength = strlen(str)+1;
   char *out = malloc(strLength);// Because \n becomes char malloc will allocate slightly more space than necessary
   char buf[2];
   memset(buf, '\0', sizeof(buf));

   while (*str != '\0')
	{
	  found = 0;
	  if(*str=='\\'){
		  str++;
		  if(*str=='n')
		     found = 1;
		  else {
		     found = 0;
		     str--;
		  }
	  }
          if(found){
	    if(firstTime){
		    strcpy(out,"\n");
		    firstTime = 0;
	    }else
		    strcat(out,"\n");
	  }else{
		strncpy(buf, str, 1);
		if(firstTime){
			strcpy(out,buf);
			firstTime = 0;
		}else
			strcat(out,buf);
	  }
   str++;
   }
return out;
}

static char *sendRequest (char *request) 
{ 
static int sequence = 1;
isdHeader header;
memset ((char *) &header, 0, sizeof header); // Initialise everything inside header to zero
header.reply = isdClientData.reply;  
header.error = 0;
header.more = isdClientData.keep;  
header.sequence = sequence++;    
#ifdef HAVE_LIBWWW
    if (unescapeURLs) HTUnEscape (request);
#endif	
if (isdClientData.debug)    
	fprintf (stderr, "isdSend()ing [%s]\n", request);
/* Send the request */  
if (isdSend (isdClientData.sock, request, -1, &header) == -1)
	perror ("isdSend()ing");
return 0;
}

/*
 * get a message from isectd.  Return the value
 * of header.more, meaning:
 * !0 = there's more to get and
 * 0 = there's no more to get.
 */ 
  
static void
getResponse (isdHeader * header) 
{
  char *msgBuffer;
  header->len = header->more = header->error = 0;

  if (msgBuffer = isdRecv (isdClientData.sock, header)){
      if (header->error == ISDERROK)
	  fwrite (msgBuffer, strlen (msgBuffer) <  header->len ? strlen (msgBuffer) : header->len, 1, stdout); 
      else
	  fputs ("There was an error processing the message.\n", stderr);   
  }else{     
	  perror ("isdRecv()");   
	  fputs ("The daemon may have terminated.\n", stderr);    
	  exit (1);   
  }
}

/*
 * getRequest() reads input from isdClientData.inputFile and
 * looks for the "go" at the end, indicating the previous
 * stuff should be sent.  When getRequest() sends its
 * message to isectd, it returns non-zero.
 */ 
  
static int getRequest (void) 
{
int size, newsize, sentRequest = 0; 
char *command;
static char *request = NULL;  
char inputBuffer[ISDCLIENT_BUFSIZE];
  
if (request == NULL)  
	*(request = malloc (size = ISDCLIENT_BUFSIZE)) = 0;
 
// Read the inputfile 
command = fgets (inputBuffer, ISDCLIENT_BUFSIZE, isdClientData.inputFile); 
if (command == inputBuffer)
    {        
    /*
     * It would be nice if it didn't have to be newline terminated, but 
     * instead could be null terminated or just a go on a line by 
     * itself, but I suppose I'd have to trim the right-side of the 
     * string of all whitespace characters.  I'm too lazy to do that 
     * today.  I'd rather write comments into my code.
     */ 
     if (strcmp (inputBuffer, "go\n") == 0)
	{
	sendRequest (request);
	sentRequest = 1;  
	*request = 0;	
     }else{
	/* Increase size of buffer until "go\n" found */	  
	newsize = (((strlen (request) + strlen (inputBuffer)) / 4096) + 1) * 4096;  
	if (newsize > size)
	    request = realloc (request, size = newsize);	  
	strcat (request, inputBuffer);	
     }     
     /*
       if (isdClientData.sendOnEOF)
       sendRequest(request);
     */ 
    }
return sentRequest;
}

static int
clientInit (int argc, char **argv) 
{
int i, c, usage = 0;
extern char *optarg;
extern int optind, opterr, optopt;
#ifndef _Windows
    setbuf (stdin, NULL);
#endif  
#ifdef HAVEWWW
    HTLibInit (PACKAGE, VERSION); 
#endif
    
isdClientData.inputFile = stdin; 
isdClientData.reply = 1;
isdClientData.service = ISDSERVICE; //Default is op service
  
while ((c = getopt (argc, argv, "c:dg:h:i:kp:ru")) != -1)
    {      
	switch (c)
	{
	// Run a single command	
	case 'c':	  
		if (optarg)
	    	{	      
			isdClientData.runOneCommand = 1;	      
			isdClientData.command = parseCommandLine(optarg);	    
		}
	  	else
	    		usage = 1;	  
	break;
	// Print command line arguments
	case 'd':	  
		isdClientData.debug = 1;
		for (i = 0; i < argc; i++)	    
			fprintf (stderr, "argv[%d] = [%s]\n", i, argv[i]);  
		fprintf (stderr, "optind = %d\n", optind);
	break;
	// Use GUI console
	case 'g':	  
		if (optarg)
			/* W Robert Kellock 2002/12/11 should be atoi not (int) which is undefined */	    
			isdClientData.guiX = atoi(optarg);	  
	  	else	    
			usage = 1;	  
	break;
	// Define host:port
	case 'h':	  
		if (optarg)	    
			isdClientData.host = optarg; 
	  	else    
			usage = 1;	  
		break;
	// Name of inputfile
	case 'i':	  
		if (optarg)
	    	{      
		  fprintf (stderr, "Reading from '%s'\n", optarg);      
		  if (!(isdClientData.inputFile = fopen (optarg, "r")))
		  {	  
		    perror (optarg);		  
		    usage = 1;		
		  }
		  /* else
		   isdClientData.sendOnEOF = 1; */ 
	    	}
	  	else   
		  usage = 1;  
		break;
	// Open a long running transaction from the client (not sure why you would do this?)
	case 'k':	  
		isdClientData.keep = 1;  
		break;
	// Set the client priority
	case 'p':  
		if (optarg)    
			isdClientData.priority = atoi(optarg);  
	  	else	    
			isdClientData.priority = 2;	  
		break;	
	// Don't wait for a reply from the worker
	case 'r':  
		isdClientData.reply = 0;	  
		break;
	// 
	case 'u':	  
		isdClientData.unescapeURLs = 1;	  
		break;
	
	default:	  
		usage = 1;	  
		break;	
}   
}
//Last command line argument is the service to attach to
if (optind < argc)    
	isdClientData.service = argv[optind];
  
return usage;
}

/* Now we begin */

#define CLIENTFILENO fileno(isdClientData.inputFile)
  
#ifdef _Windows
  int APIENTRY WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow){ 
#else	
  int main (int argc, char *argv[]){
#endif
  
int status, mask, receivedReply = 1;
char *request; 
fd_set readfds, allfds;
  
#ifdef _Windows	
  int j, argc; 
  char **argv;
  argv = CommandLineToArgvWR (GetCommandLine (), &argc);
#endif

isdClientData.priority = 2; /* default priority */
if (clientInit (argc, argv))
   fprintf (stderr,
		 "%s [-c command] [-d] [-g consolewidth] [-h host[:port]] [-i inputfile] [-k] [-p priority] [-r] [servicename]\n",
		 argv[0]);
else
   {
      if (isdClientData.guiX){
	#ifdef _Windows
		RedirectIOToConsole (isdClientData.guiX);
	#endif
	}	
     if ((isdClientData.sock =
	    isdLogin (isdClientData.host, isdClientData.service,
		      isdClientData.priority)) != INVALID_SOCKET)
	{	  
	if (isdClientData.runOneCommand)
   		{	      
   		sendRequest (isdClientData.command);
   		receivedReply = 0;
		/* W Robert Kellock 2003/11/27 -r flag and -c flag used together */
		if (isdClientData.reply == 0)
		   {
		   fflush (stdout);			  
		   isdLogout (isdClientData.sock);			  
		   exit (1);
		   }
   		}

FD_ZERO (&allfds);
FD_SET (CLIENTFILENO, &allfds);
FD_SET (isdClientData.sock, &allfds);	  

for (;;)
  {

  /* 
     W Robert Kellock 2002/08/24 select() cannot
     use file handles under Windows so instead
     we are using a combination of CLIENTFILENO == 0
     and receivedReply which is initialised as 1 when
     it was declared.
   */ 
  if (feof (isdClientData.inputFile) || (CLIENTFILENO == 0))		
	FD_CLR (CLIENTFILENO, &allfds);
	      

  for (status = -1; status == -1;)
		{  
		readfds = allfds;	  
		mask = max (CLIENTFILENO, isdClientData.sock);		  
		if (receivedReply == 0)
		    {
		      status =	select (mask + 1, &readfds, (fd_set *) 0,
				(fd_set *) 0, (struct timeval *) 0);
		      if (status == SOCKET_ERROR && errno != EINTR)
			{		  
			isdError ("select()ing\n");			  
			exit (1);			
			}      
			else
			{	  
			/* 
			   W Robert Kellock 2002/08/28 
			   run one command (-c) is true
			   under Windows
			*/
			fflush (stdout);
			isdLogout (isdClientData.sock);
			exit (1);			
			}
		    }		  
		else  
		    break;
		}

	if (FD_ISSET (CLIENTFILENO, &readfds) || (CLIENTFILENO == 0))
		{
		 /* 
		    W Robert Kellock 2002/08/24 select() cannot
		    use file handles under Windows so instead we
		    look at getRequest() every loop.
		  */ 
		 if (getRequest ())
		 {
		    FD_CLR (CLIENTFILENO, &allfds);	      
		    receivedReply = 0;	    
		 }
		/*
		 * After getting a request from the operator mark that I'm waiting for a reply.
		 * It's possible that I didn't get a request but instead got an EOF.  In which
		 * case if I'm not waiting, terminate the loop.
		 */ 
		 if (receivedReply && feof(isdClientData.inputFile)){	    
			break;
		}		
	}	      
       /*
	* handle responses from the worker (or isectd).
	*/ 
       if (FD_ISSET (isdClientData.sock, &readfds) && (receivedReply == 0))
       {	  
	isdHeader header;
	getResponse (&header);	  
       /*
	* The reason I'm !ing header.more, is I need to know if I need to keep
	* waiting for another reply.  If there's more, than I need to keep waiting.  If
	* there isn't any more (header.more == 0) then I've received everything
	* I'm supposed to... at least as far as the worker is concerned.
	*/ 
	while (header.more)
		getResponse (&header);   
	receivedReply = !header.more;
       /*
	* If I've received a reply, then I can start listening to the client
	* input file again in the select() loop.
	*/ 
	if (receivedReply)		    
		FD_SET (CLIENTFILENO, &allfds);		  
       /*
	* I want to stop processing if I've gotten my reply and I'm only supposed to
	* run one command (-c) or the input file is EOF, or there was some kind of
	* error processing the command (header.error != ISDERROK)
	*/ 
	if (receivedReply && (isdClientData.runOneCommand||feof(isdClientData.inputFile)||header.error != ISDERROK))		    
		break;		
  }  
}  
fflush (stdout);	  
isdLogout (isdClientData.sock);	
}      
	else	
		perror ("isdLogin() has failed.");    
}
  
#ifdef HAVEWWW
    HTLibTerminate ();  
#endif	
return 0;
}




