/*
W Robert Kellock 2014/02/27 a simple client designed to be used
with the paraEcho worker. It's purpose to flood the Isectd server
with multiple requests for parallel processing. Have added sequence 
to header to see the replies are returning in an arbitrary order.
It does not expect to receive a reply from the paraEcho worker,
using Fire and Forget messaging.

Usage would be:

paraSend -h localhost:5501 -s service -r 200 -c "The string that we want to echo\n"
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <malloc.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "../isd/isdio.h" /* W Robert Kellock 2002/12/11  hard path */
/* #include "../vb/VBisdio.h" */

#define ISDCLIENT_BUFSIZE 4096

static struct
{
int repeat;
char *command;
char *host;
char *service;
}
isdClientData;

char *parseCommandLine(char *str){
   /* 
	Convert \n's anywhere in command line arguments to newlines
   */
   int i, found, firstTime = 1;
   int strLength = strlen(str)+1;
   char *out = malloc(strLength);// Because \n becomes char malloc will allocate slightly more space than necessary
   char buf[2];
   memset(buf, '\0', sizeof(buf));

   while (*str != '\0')
	{
	  found = 0;
	  if(*str=='\\'){
		  str++;
		  if(*str=='n')
		     found = 1;
		  else {
		     found = 0;
		     str--;
		  }
	  }
          if(found){
	    if(firstTime){
		    strcpy(out,"\n");
		    firstTime = 0;
	    }else
		    strcat(out,"\n");
	  }else{
		strncpy(buf, str, 1);
		if(firstTime){
			strcpy(out,buf);
			firstTime = 0;
		}else
			strcat(out,buf);
	  }
   str++;
   }
return out;
}

int clientInit (int argc, char **argv)
{
int c;
int usage = 0;

extern char *optarg;
extern int optind, opterr, optopt;

#ifndef _Windows
	setbuf(stdin, NULL);
#endif

while ((c = getopt (argc, argv, "c:h:r:s:")) != -1){
    switch (c)
	{
    case 'c':
	  if (optarg){
         isdClientData.command = parseCommandLine(optarg);
         usage++;
         }
      break;
    case 'h':
	  if (optarg){
         isdClientData.host = optarg;
         usage++;
         }
      break;
    case 'r':
	  if (optarg){
         isdClientData.repeat = atoi(optarg);
         usage++;
         }
      break;
    case 's':
      if (optarg){
         isdClientData.service = optarg;
         usage++;
         }
      break;
      }
    }
return usage;
}

int
main(int argc, char **argv)
{

	SOCKET sock;
	long sequence;
	isdHeader header;
	char msgBuffer[ISDCLIENT_BUFSIZE];
	char msgSequence[ISDCLIENT_BUFSIZE];

if (clientInit(argc, argv) < 4)
   fprintf (stderr,
	      "%s [-h host[:port]] [-r loop] [-s servicename] [-c command] \n",
	      argv[0]);
else {
  /*
  While a single client can parallel process, the only way to do it is to trick Isectd into thinking it is dealing 
  with multiple clients by logging in for each iteration and immediately logging off. Otherwise with header.reply = 0,
  Isectd assumes that it is dealing with a dead client and queues all that client's sends to the same worker.  This is 
  by design, because if a client has died you don't want it's potentially dangerous data endangering the remaining workers.
  */
  for (sequence = 1; sequence < isdClientData.repeat + 1; sequence++) {
	if ((sock = isdLogin(isdClientData.host, isdClientData.service, 2)) != INVALID_SOCKET) {
		memset((char *) &header, 0, sizeof header);
		header.reply = 0; // Dump any replies from the worker... Fire and forget!
		header.error = 0;
		header.more  = 0;
		header.unused[1] = 1;  // See paraEcho
		header.sequence = sequence;
		if (isdSend(sock, isdClientData.command, -1, &header) != -1){} else perror("isdSend()ing");
	        fflush(stdout);
	        isdLogout(sock);
	}
	else{
		perror("isdLogin()");
		exit(0);
	}
  }
}
	return 0;
}

