#ifndef ISDIO_H
#define ISDIO_H

#define ISDHOSTNAME "isectd"
#define ISDSERVICE "op"

#include <mingw.h>  /* W Robert Kellock 2002/12/18 soft path */
/* W Robert Kellock 2002/08/09 Added Windows switch */
#ifndef _Windows
#include <sys/time.h>
#include <sys/types.h>
#include <sys/param.h>
#endif

#include <stdarg.h>
#include <stdio.h>

#ifdef _Windows
#include <windows.h>  /* W Robert Kellock 2002/08/09 windows.h replaces winsock.h */
#include <process.h>
#include <io.h>
#ifndef _WIN32
#define getpid() 0
#endif

/* 
   W Robert Kellock 2002/08/15 
   Build DLL, use DLL or static library
*/
#if BUILDING_DLL
#define Xport __declspec (dllexport)
#elif STATIC_LIB
#define Xport _export far pascal
#else
#define Xport __declspec (dllimport)
#endif

#else
#define SOCKET int
#define INVALID_SOCKET -1
#define SOCKET_ERROR -1
#define O_BINARY 0

#include <sys/socket.h>
#include <netinet/in.h>

/*
most systems have netinet/tcp.h, but MVS
systems have xti.h instead, which defines
things like TCP_NODELAY
*/

#ifdef HAVE_NETINET_TCP_H
#include <netinet/tcp.h>
#endif

#ifdef HAVE_XTI_H
#include <xti.h>
#endif

#include <netdb.h>
#define Xport
#endif

#ifndef _H_INTTYPES
#ifndef __BIT_TYPES_DEFINED__
#define __BIT_TYPES_DEFINED__
typedef unsigned short  u_int16_t;
typedef unsigned int    u_int32_t;
#ifndef _STDINT_H /* Robert Kellock 15/11/2004 */
typedef short			int16_t;
typedef int				int32_t;
#endif
#endif
#endif

#ifndef MAXSERVICENAMELEN       /* until I can find one in an include file, I'll define my own */
#define MAXSERVICENAMELEN 30    /* W Robert Kellock 2017/09/24 reduced from 32 to 30 to fit in isdBadgeStr */
#endif
#ifndef MAXPATHLEN              /* W Robert Kellock 2002/08/09 shifted to here from rmt.h */
#define MAXPATHLEN 1024
#endif


/*
The arguments to isdAttach() are in a specific order on the
worker's command line.
*/
#define ISDARGHOST		1			/* the hostname I'm calling back to */
#define ISDARGSERVICE	2				/* the service I'm a worker for */
#define ISDARGWID		3			/* my worker id, how isectd identifies me */
#define ISDARGMAX		4			/* worker's argc should be >= ISDARGMAX */

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Errors retured in header.error
 */
#define ISDERROK 0
#define ISDERRWORKERDEATH 1

typedef struct  {		
	int32_t	len;		
	int32_t	sequence;	
	int16_t	reply;		
	int16_t	error;		
	int16_t	command;	
	int16_t	version;	
	u_int32_t workerid; 	
	int16_t	more;		
	int16_t	unused[17];
	double aligner;
} isdHeader; /* 48 bytes */

typedef struct  {
        int32_t id; 	    /* required of workers */
        int32_t hostpid;    /* required of both workers and clients */
        int32_t priority;   /* required of clients */
        int32_t blockable;  /* tells isectd whether this process can be blocked */
	char service[MAXSERVICENAMELEN];
} isdBadge; /* 48 bytes */

/*  
	W Robert Kellock 2017/09/24 string marshalling structs to avoid alignment and padding problems. Each
	char array is assumed to contain at least one trailing '\0' so char[3] could range from -9 to 99
*/

typedef struct  {		
	char  len[7];	
	char  reply[3];	
	char  more[3];		
	char  sequence[9];	
	char  error[3];		
	char  command[3];	
	char  version[3];	
	char  workerid[8]; 	
	char  unused[9];
} isdHeaderStr; /* 48 bytes */

typedef struct  {
        char hostpid[7];    /* required of both workers and clients */
        char priority[3];   /* required of clients */
        char id[5]; 	    /* required of workers */
        char blockable[3];  /* tells isectd whether this process can be blocked */
	char service[MAXSERVICENAMELEN];
} isdBadgeStr; /* 48 bytes */


#define ISDXSERVICE "isdexec"

/*  W Robert Kellock 2002/08/17 removed extern operator */
char * Xport isdxstrerror(int);
Xport int	isdexec(char *host, char *command);

Xport void isdLogInit(int argc, char **argv);
Xport void isdLog(char *fmt, ...);
Xport void isdError(char *fmt, ...);

Xport SOCKET isdAttach(const char *hostname, const char *service, char *wid, int priority);
Xport SOCKET isdConnectTo(const char *hostname, const char *service);
Xport SOCKET isdLogin(const char *hostname, const char *service, int priority);
Xport void  isdLogout(SOCKET sock);
char * Xport isdRecv(SOCKET socket, isdHeader *header);
Xport int isdRecvHeader(SOCKET socket, isdHeader * header);
Xport int isdRecvBuffer(SOCKET socket, isdHeader *header, char *buffer, int bufferLength);
Xport int isdRecvAll(SOCKET socket, char *buffer, int len);
Xport int isdSend(SOCKET socket, char *message, int len, isdHeader *header);
Xport SOCKET isdConnect(char **argv);
Xport SOCKET isdSocket(char *name, char *protocol, SOCKET *sock, int *port);
Xport void   isdSockOpts(SOCKET sock);

#ifdef __cplusplus
}
#endif

#endif
