#include <stdlib.h>
#include <errno.h>
/*
#include <pthread.h>  W Robert Kellock 2002/08/09
*/

#include "isdio.h"
#ifdef _Windows
#define EWOULDBLOCK WSAEWOULDBLOCK
#endif

void isdInHeader(isdHeaderStr *headerStr, isdHeader *header)
{
	/* 
		Convert incoming string array to the native isdHeader struct

		Might need some checks in here to avoid copying in dud data 
	*/
	header->len = atoi(headerStr->len);
	header->reply = atoi(headerStr->reply);
	header->more = atoi(headerStr->more);
	header->sequence = atoi(headerStr->sequence);
	header->error = atoi(headerStr->error);
	header->command = atoi(headerStr->command);
	header->version = atoi(headerStr->version);
	header->workerid = atoi(headerStr->workerid);
}

Xport int isdRecvAll(SOCKET socket, char *buffer, int len)
{
	int bytesSoFar, bytes;

	for (bytesSoFar = 0; bytesSoFar < len; bytesSoFar += bytes) {
		do {
			bytes = recv(socket, &buffer[bytesSoFar], len - bytesSoFar, 0);
		} while (bytes == -1 &&
			(errno == EINTR || errno == EWOULDBLOCK || errno == EAGAIN));

		if (bytes <= 0)
			break;
	}

	return (bytes == -1) ? -1:bytesSoFar;
}


Xport int isdRecvHeader(SOCKET socket, isdHeader * header)
{
	int bytes;
	static char *headerBuffer = NULL; /* Static as sizeof *headerStr will never change */
	isdHeaderStr *headerStr;

	if (headerBuffer == NULL)
		headerBuffer = (char *) malloc(sizeof *headerStr);

	bytes = isdRecvAll(socket, headerBuffer, sizeof *headerStr);
	headerStr = (isdHeaderStr *) headerBuffer;
	isdInHeader(headerStr, header);

	//free(headerBuffer);
	return bytes == sizeof(*headerStr) ? 1:0;
}

char * Xport isdRecv(SOCKET socket, isdHeader * header)
{
	int len, bytes;
	char *retbuf = NULL;
	static int size = 0;
	static char *buffer = NULL;

	if (buffer == NULL)
		buffer = (char *) malloc(size = 4096);

	if (isdRecvHeader(socket, header)) {

		if ((len = header->len) > size)
			buffer = (char *) realloc((void *) buffer, size = len);

		bytes = isdRecvAll(socket, buffer, len);

		if (bytes == len)
			retbuf = buffer;
	}

	return retbuf;
}

/*
	W. Robert Kellock 2017-09-25

	isdRecvBuffer function now deprecated since the implementation of marshalling C char arrays 
	over sockets rather than C structs means that clients and workers can now communicate
	with Isectd directly rather using this API.
*/

/*
	This function provides a mechanism for programs that want to handle
	memory management on their own (like Smalltalk VMs) or multi-threaded
	programs that can't share an internal buffer.  

	If the buffer supplied is large enough then the packet is read into
	it.  If the buffer is not, the packet payload is discarded (for now).
*/

int Xport
isdRecvBuffer(SOCKET socket, isdHeader *header, char *buffer, int bufferLength)
{
	int bytes;
	char *localBuffer;

	if (isdRecvHeader(socket, header)) {
		localBuffer = buffer;
		if (bufferLength < header->len)
			localBuffer = malloc(header->len);

		bytes = isdRecvAll(socket, localBuffer, header->len);

		if (localBuffer != buffer)
			free(localBuffer);
	}

	return bytes;
}
