#ifndef QUEUES_H
#define QUEUES_H

#include "libisect.h"

typedef struct Node {
	struct Node *next;
	struct Node *previous;
	struct Queue *queue;
} Node;

typedef struct Queue {
	Node *head;
	Node *tail;
	long count;
} Queue;

#define QUEUE(name, type) struct {type *head; type *tail;} name;
#define NODE(type) struct {type *next; type *previous; void *q;} node;

#define QNULL (void *) -1
#define QEQ 1
#define QGT 2
#define QLT 3
#define QGE 4
#define QLE 5
#define QNE 6
#define QAND 7
#define QNAND 8
#define QOR 9
#define QXOR 10

/* W Robert Kellock 2002/08/17 removed all extern operators */
Xport Node* InsertAfter(Node *, Node *);
Xport Node* InsertBefore(Node *, Node *);
Xport Node* InsertHead(Queue *, Node *);
Xport Node* InsertTail(Queue *, Node *);
Xport Node* RemoveHead(Queue *);
Xport Node* RemoveTail(Queue *);
Xport Node* RemoveNode(Node *);
Xport Queue* NodeFree(Queue *);
Xport Queue* QueueFree(Queue *);
Xport Queue* QueueAlloc(int);
Xport Queue* QueueInit(Queue *);
Xport int    QueueItems(Queue *);
/* W Robert Kellock 2003/02/17 changed int to long */
Xport Node*  QueueSearch(Queue *, int, int, long, Node *);
Xport Node*  QueueByteCmp(Queue *, int, int, char *, int, Node *);

#endif
