/*
W Robert Kellock 2014/02/27 

A worker that accumulates data sent to it into a queue if header.unused[0] is true.  Otherwise it 
empties the queue and returns all queued data to the requesting client.

*/
#include <stdio.h>
#include <errno.h>
#include "../isd/isdio.h"  /* W Robert Kellock 2002/08/09 hard path */
#include "../lib/queues.h"

int main(int argc, char **argv)
{
	int bytes;
	SOCKET sock;
	isdHeader header;
	char *msgBuffer;

	typedef struct {
		Node node;
		long sequence;
		char *msgBuffer;
	} msgFragment;
	msgFragment *thisFragment;

	Queue allFragments;
	QueueInit(&allFragments);

	isdLogInit(argc, argv);

	if (argc >= ISDARGMAX) {
		if ((sock = isdConnect(argv)) != INVALID_SOCKET) {
		  	for (;;) {
				if (msgBuffer = isdRecv(sock, &header)) {
					/* Using header.unused is potentially dangerous.  It depends on header 
					being created by the client with calloc or filled with zeros using memset */
					if (header.unused[0]){
					       /* paraEcho worker is sending data to be accumulated */
						thisFragment = (msgFragment *) malloc(sizeof *thisFragment);
  						thisFragment->sequence = header.sequence;
						thisFragment->msgBuffer = malloc(header.len);
						memcpy(thisFragment->msgBuffer, msgBuffer, header.len);
						InsertHead(&allFragments, (Node *) thisFragment);
						// Need to reply to free paraEcho worker to accept another client
						bytes = isdSend(sock, "Junk", 4, &header);
				       }else{
					       /* isdClient is requesting all data be returned to it */
						while ((thisFragment = (msgFragment *)RemoveTail(&allFragments)) != QNULL){
							header.sequence = thisFragment->sequence;
							header.more = 1;
							header.reply = 1;
							bytes = isdSend(sock, thisFragment->msgBuffer, strlen(thisFragment->msgBuffer), &header);
							free(thisFragment->msgBuffer);
							free(thisFragment);
						}
						msgBuffer = "Finished.\n";
						header.more = 0;
						header.reply = 1;
						bytes = isdSend(sock, msgBuffer, strlen(msgBuffer), &header);
				       }
				       if (bytes < 0)
					       isdError("%s: isdSend() reply\n", argv[ISDARGWID], strerror(errno));
				}else
					break;
			}
			isdLog("Worker %s exiting\n", argv[ISDARGWID], strerror(errno));
			isdLogout(sock);
	      	}else
		       isdError("isdConnect(): %s\n", strerror(errno));
	}
	else
		fprintf(stderr, "Incorrect number of arguments\n");
	return 0;
}
