/*
W Robert Kellock 2014/03/04 

paraEcho.c

A worker AND client designed to be used with the paraAccum worker.
It passes messages received from the paraSend client to the paraAccum
worker as soon as they are received.  It needs two arguments,

1.  Isectd's host:port
2.  The Service that paraAccum is attached to.

It can also be used with isdClient which will display the echoed message.

header.unused[0] = 1 tells paraAccum to queue data
header.unused[1] == 1 tells paraEcho to echo the message back to isdClient
header.unused[1] == 0 tells Isectd to dump the message as it came from paraSend which doesn't want a reply
*/
#include <stdio.h>
#include <errno.h>
#include "../isd/isdio.h"  /* W Robert Kellock 2002/08/09 hard path */

int main(int argc, char **argv)
{
	int bytes;
	SOCKET sock1, sock2;
	isdHeader header;
	char *msgBuffer;
	char *out; 

	isdLogInit(argc, argv);

	if (argc >= ISDARGMAX + 2) {
		if ((sock1 = isdConnect(argv)) != INVALID_SOCKET) {
		  if ((sock2 = isdLogin(argv[ISDARGMAX + 1], argv[ISDARGMAX + 2], 2)) != INVALID_SOCKET) {
			for (;;) {
				if (msgBuffer = isdRecv(sock1, &header)) {
					/* Prepend msgBuffer with sequence number */
					header.len = header.len + 33 + 1;
					out = malloc(header.len);
					sprintf(out, "%d", header.sequence);
					strcat(out, " ");
					strcat(out, msgBuffer);
					/*
						so paraAccum knows to accumulate data we are using header.unused[0]
					*/
					header.unused[0] = 1;
					header.more = 0;
					header.reply = 0; // Dump any replies from the worker... Fire and forget!
					bytes = isdSend(sock2, out, header.len, &header);
					if (bytes < 0)
						isdError("%s: isdSend() reply\n", argv[ISDARGWID], strerror(errno));
					// Pretend this task takes a while so that Isectd is forced to allocate to it's
					// worker pool equally. 
					Sleep(10); // 10 milliseconds
					// Need to reply to original client to free this worker to accept another client
					header.unused[0] = 0;
					header.more = 0;
					if(header.unused[1])
						header.reply = 0; // paraSend
					else
						header.reply = 1; // isdClient
					bytes = isdSend(sock1, out, header.len, &header);
					free(out);
					if (bytes < 0)
						isdError("%s: isdSend() reply\n", argv[ISDARGWID], strerror(errno));
				}
				else
					break;
			}
			isdLog("Worker %s exiting\n", argv[ISDARGWID], strerror(errno));
			isdLogout(sock2);
			isdLogout(sock1);
		  }
		  else
			isdError("isdLogin(): %s\n", strerror(errno));
	      	}else
		       isdError("isdConnect(): %s\n", strerror(errno));
	}
	else
		fprintf(stderr, "Incorrect number of arguments.\nIsectd's host:port\nThe service you are attaching to.");
	return 0;
}
